@@ -23,6 +23,7 @@ Coro::Cont - schmorp's faked continuations
 package Coro::Cont;
 
 use Coro::State;
+use Coro::Specific;
 
 use base 'Exporter';
 
@@ -35,20 +36,17 @@ Create a new "continuation" (well, almost, you cannot return from it).
 
 =cut
 
-our $prev = new Coro::State;
-our $cont;
-our $result;
+our $curr = new Coro::Specific;
+our @result;
 
 sub cont(&) {
    my $code = $_[0];
-   my $coro = new Coro::State sub {
-      &$code while 1;
-   };
+   my $coro = new Coro::State sub { &$code while 1 };
+   my $prev = new Coro::State;
    sub {
-      local $cont = $coro;
-      local $result;
-      $prev->transfer($cont);
-      @$result;
+      push @$$curr, [$coro, $prev];
+      $prev->transfer($coro);
+      wantarray ? @{pop @result} : ${pop @result}[0];
    };
 }
 
@@ -59,21 +57,14 @@ Return the given list/scalar as result of the continuation.
 =cut
 
 sub result {
-   $result = [@_];
-   $cont->transfer($prev);
+   push @result, [@_];
+   &Coro::State::transfer(@{pop @$$curr});
 }
 
 1;
 
 =back
 
-=head1 BUGS
-
-This module does not yet work in the presence of coroutines (see
-L<Coro>). The reason for this is that there is no defined way to
-save/restore globals on task-switches. Implementing Coro::Cont using
-Coro would fix this, but is rather overkilled ;)
-
 =head1 AUTHOR
 
  Marc Lehmann <pcg@goof.com>
